/**
 * Title:       GUI MQ class for ASETS project (ASE Trading System)
 * Author:      Claudiu Vinte (claudiu.vinte@ase.ro) and others
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Provides a convenient API for GUI to easily communicate with
 * other modules through a MQ broker
 */

package asets.gui.communication;
import asets.api.*;
import asets.gui.application.TradingGUI;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.ObjectMessage;


public class GUIExecutionSubscriber extends ExecutionSubscriber {
    TradingGUI tradingGUI;

    public GUIExecutionSubscriber(TradingGUI parent) throws JMSException, Exception {
        super(TradingGUI.IMQ_ADDRESS, Enums.TopicName.CLIENT_EXECUTION_TOPIC.toString(),
                parent.getLoggedUser().getUserID() + "-ClientExecution");
        
        tradingGUI = parent;
        System.out.println("Subscribing for executions...");
    }

    // Receive messages from Execution topic
    @Override
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if ( message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.EXECUTION.toString()))
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;
                    Execution receivedExecution = (Execution)objectMessage.getObject();

                    if (receivedExecution.getUserID().equals(this.tradingGUI.getLoggedUser().getUserID()))
                        processExecution(receivedExecution);
                }
            }
        } catch (JMSException e) {
                e.printStackTrace();
        } catch (Exception e) {
                e.printStackTrace();
        }
    }
    
    @Override
    protected void processExecution(Execution execution) throws Exception {
        // TODO : add or destroy
        System.out.println("ExecutionSubscriber processing execution - " + execution.getExecID()
                + ". Execution OrderID - " + execution.getOrderID());
        this.tradingGUI.addExecution(execution);
        // TODO : update order in panel
    }

    public TradingGUI getTradingGUI() {
        return tradingGUI;
    }
}
