/**
 * Title:       GUI Order topic subscriber with symbol filtering, for ASETS project (ASE Trading System)
 * ClassName:   GUIInstrumentOrderSubscriber.java
 * Created:     September 12, 2011
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Order topic subscriber with symbol filtering within OpenMQ broker, by extending OrderSubscriber class
 */

package asets.gui.communication;

import asets.api.*;
import asets.gui.application.TradingGUI;
import asets.gui.frames.MarketDepthFrame;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.ObjectMessage;


public class GUIInstrumentOrderSubscriber extends OrderSubscriber 
{
    private TradingGUI tradingGUI = null;
    private String symbol = null;
    private MarketDepthFrame frame = null;

    public GUIInstrumentOrderSubscriber(TradingGUI applet, MarketDepthFrame frame, String symbol) 
            throws JMSException, Exception 
    {
        super(TradingGUI.IMQ_ADDRESS, 
            Enums.TopicName.CLIENT_ORDER_TOPIC.toString(),
            applet.getLoggedUser().getUserID() + "-ClientOrder-" + symbol);
        
        this.tradingGUI = applet;
        this.frame = frame;
        this.symbol = symbol;
        System.out.println("Subscribing for orders filtered by symbol...");
    }

    // Receive messages from Order topic
    @Override
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if ( message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.ORDER.toString()))
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;
                    Order receivedOrder = (Order)objectMessage.getObject();

                    if (receivedOrder.getSymbol().equals(symbol))
                        processOrder(receivedOrder);
                }
            }
        } catch (JMSException e) {
                e.printStackTrace();
        } catch (Exception e) {
                e.printStackTrace();
        }
    }

    @Override
    protected void processOrder(Order order) throws Exception 
    {
        System.out.println("InstrumentOrderSubscriber processing order - " + order.getOrderID());
        this.frame.receivedOrder(order);
    }

    public TradingGUI getTradingGUI() {
        return tradingGUI;
    }
}