/**
 * Title:       GUI Order topic subscriber, for ASETS project (ASE Trading System)
 * ClassName:   GUIOrderSubscriber.java
 * Created:     September 12, 2011
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Order topic subscriber within OpenMQ broker, by extending OrderSubscriber class
 */

package asets.gui.communication;

import asets.api.*;
import asets.gui.application.TradingGUI;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.ObjectMessage;


public class GUIOrderSubscriber extends OrderSubscriber 
{
    private TradingGUI tradingGUI = null;
   
    public GUIOrderSubscriber(TradingGUI applet) throws JMSException, Exception 
    {
        super(TradingGUI.IMQ_ADDRESS, Enums.TopicName.CLIENT_ORDER_TOPIC.toString(),
            applet.getLoggedUser().getUserID() + "-ClientOrder");
        
        this.tradingGUI = applet;
        System.out.println("Subscribing for orders...");
    }

    // Receive messages from Order topic
    @Override
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if ( message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.ORDER.toString()))
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;
                    Order receivedOrder = (Order)objectMessage.getObject();

                    if (receivedOrder.getUserID().equals(this.tradingGUI.getLoggedUser().getUserID()))
                        processOrder(receivedOrder);
                }
            }
        } catch (JMSException e) {
                e.printStackTrace();
        } catch (Exception e) {
                e.printStackTrace();
        }
    }
    
    @Override
    protected void processOrder(Order order) throws Exception 
    {
        System.out.println("OrderSubscriber received order - " + order.getOrderID()
                + ". Order Status = " + order.getOrderStatus()+"; Market Status = "+order.getMarketStatus().toString());
        this.tradingGUI.addOrUpdateOrder(order);
    }

    public TradingGUI getTradingGUI()
    {
        return tradingGUI;
    }
}
