/**
 * Title:       GUI Helper class for ASETS project (ASE Trading System)
 * Author:      Claudiu Vinte (claudiu.vinte@ase.ro) and others
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Provides a various functionality for GUI
 */

package asets.gui.core;

import asets.api.Enums;
import asets.api.Order;
import java.util.HashMap;

/**
 *
 * @author Adi
 */
public class InstrumentPriceInfo implements Comparable<InstrumentPriceInfo>{
    private long quantity;
    private Float price;
    private long numOrders;
    private HashMap<String,Order> orders;

    public HashMap<String, Order> getOrders() {
        return orders;
    }

    public InstrumentPriceInfo(Float price) {
        quantity = 0;
        numOrders =0;
        this.price = price;
        orders = new HashMap<String, Order>();
    }

    public Boolean addOrder(Order order) {
        if (order.getOrderPrice() == price) {
            if (!orders.containsKey(order.getOrderID())) {
                 if (order.getRemainedQuant() != 0 && !order.getOrderStatus().equals(Enums.OrderStatus.CANCELED)) {
                    orders.put(order.getOrderID(), order);
                    quantity+= order.getRemainedQuant();
                    numOrders++;
                    
                    return true;
                 } else {                    
                     return false;
                 }
            } else {
                if (order.getRemainedQuant() == 0 || order.getOrderStatus().equals(Enums.OrderStatus.CANCELED)) {
                    Order searchOrder = orders.get(order.getOrderID());
                    quantity-= searchOrder.getRemainedQuant();
                    numOrders--;
                    orders.remove(order.getOrderID());
                    
                    return true;
                } else {
                    Order searchOrder = orders.get(order.getOrderID());
                    quantity-= searchOrder.getRemainedQuant();
                    quantity+= order.getRemainedQuant();
                    orders.put(order.getOrderID(), order);
                    
                    return true;
                }
            }
        } else {
            return false;
        }
    }
    
    public Boolean removeOrder(String orderID) {
        Order orderToRemove = orders.get(orderID);

        if( orderToRemove == null ) 
            return false;
        
        quantity -= orderToRemove.getRemainedQuant();
        numOrders--;       
        orders.remove(orderID);

        return true;
    }

    public long getNumOrders() {
        return numOrders;
    }

    public void setNumOrders(long numOrders) {
        this.numOrders = numOrders;
    }

    public Float getPrice() {
        return price;
    }

    public void setPrice(Float price) {
        this.price = price;
    }

    public long getQuantity() {
        return quantity;
    }

    public void setQuantity(long quantity) {
        this.quantity = quantity;
    }

    public int compareTo(InstrumentPriceInfo o) {
        return getPrice().compareTo(o.getPrice());
    }


}
