/**
 * Title:       PriceFormat for ASETS project (ASE Trading System)
 * ClassName:   PriceFormat.java
 * Created:     June 22, 2011
 * Author:      Ionut-Alexandru Lixandru (alex.lixandru@gmail.com)
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements a simple formatter class that is used across all GUI
 * 				components, providing consistency for the price formats.
 */
package asets.gui.core;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;

public class PriceFormat extends DecimalFormat
{
    private static DecimalFormatSymbols asetsFormat = null; 

    public PriceFormat() {
        super();
        PriceFormat.updateFormat(this);
    }

    public static DecimalFormatSymbols getFormatSymbols() {
        if( asetsFormat == null ) {
            asetsFormat = new DecimalFormatSymbols();
            asetsFormat.setDecimalSeparator('.');
            asetsFormat.setGroupingSeparator(' ');
        }
        
        return asetsFormat;
    }

    public static void updateFormat(DecimalFormat f) {
        f.setDecimalSeparatorAlwaysShown(true);
        f.setGroupingUsed(true);
        f.setMaximumFractionDigits(4);
        f.setMinimumFractionDigits(4);

        f.setDecimalFormatSymbols(PriceFormat.getFormatSymbols());
    }
}
