package asets.gui.frames;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.text.DecimalFormat;
import java.util.Date;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.jms.JMSException;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JSpinner.NumberEditor;

import asets.api.Enums;
import asets.api.InstrumentList;
import asets.api.Order;
import asets.gui.application.TradingGUI;
import asets.gui.communication.GUIOrderSenderWithReply;
import asets.gui.core.PriceFormat;
import asets.gui.panels.SlimStatusBar;

public abstract class AbstractOrdersFrame extends JDialog {

	private static final long serialVersionUID = -384481845288127993L;
	protected JComboBox orderTypeComboBox, symbolComboBox;
	protected JLabel orderTypeLabel, priceLabel, quantityLabel, symbolLabel;
	protected JSpinner priceSpinner, quantitySpinner;
	protected JButton sendButton;

	protected TradingGUI parentApplet;
	protected InstrumentList instrumentList;
	protected SlimStatusBar statusBar;
	protected Enums.Side side;

	
	/**
	 * Creates all form items
	 */
	protected  void initCommonComponent() {

		symbolLabel = new javax.swing.JLabel("Symbol");
		symbolLabel.setPreferredSize(new Dimension(70, 20));
		symbolComboBox = new javax.swing.JComboBox();
		symbolComboBox.setPreferredSize(new Dimension(100, 20));

		quantityLabel = new javax.swing.JLabel("Quantity");
		quantityLabel.setPreferredSize(new Dimension(50, 20));
		quantitySpinner = new JSpinner();
		quantitySpinner.setPreferredSize(new Dimension(60, 20));
		quantitySpinner.setModel(new javax.swing.SpinnerNumberModel(Long.valueOf(1), Long.valueOf(1), null, Long.valueOf(1)));
		((NumberEditor) quantitySpinner.getEditor()).getFormat().setGroupingUsed(false);
        
		orderTypeLabel = new javax.swing.JLabel("Order type");
		orderTypeLabel.setPreferredSize(new Dimension(70, 20));
		orderTypeComboBox = new javax.swing.JComboBox();
		orderTypeComboBox.setPreferredSize(new Dimension(100, 20));
		orderTypeComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Limit Price", "Market Price" }));

                priceLabel = new javax.swing.JLabel("Price");
		priceLabel.setPreferredSize(new Dimension(35, 20));
		priceSpinner = new JSpinner();
		priceSpinner.setPreferredSize(new Dimension(75, 20));
		priceSpinner.setModel(new javax.swing.SpinnerNumberModel(Float.valueOf(1.0f), Float.valueOf(0.0001f), null, Float.valueOf(0.01f)));
		JSpinner.NumberEditor ne = new JSpinner.NumberEditor(priceSpinner);
		PriceFormat.updateFormat(ne.getFormat());
		priceSpinner.setEditor(ne);
		ne.getTextField().setText( ne.getFormat().format(1) ); // hack to get the right format displayed the first time
		
		sendButton = new javax.swing.JButton("Send");
		sendButton.setMnemonic('s');

		sendButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				sendOrder(side);
			}
		});

		((JSpinner.DefaultEditor)quantitySpinner.getEditor()).getTextField().addKeyListener( new KeyListener() {
			public void keyTyped(KeyEvent evt) { keyTypedHandler(evt); }
			public void keyPressed(KeyEvent arg0) { /* no implementation */ }
			public void keyReleased(KeyEvent key) { /* no implementation */ }
		});

		((JSpinner.DefaultEditor)priceSpinner.getEditor()).getTextField().addKeyListener( new KeyListener() {
			public void keyTyped(KeyEvent evt) { keyTypedHandler(evt); }
			public void keyPressed(KeyEvent arg0) { /* no implementation */ }
			public void keyReleased(KeyEvent key) { /* no implementation */ }
		});

		JPanel topContainer = new JPanel();
		topContainer.setSize(380,160);
		topContainer.setPreferredSize(new Dimension(380, 160));
		topContainer.setLayout(new BoxLayout(topContainer, BoxLayout.PAGE_AXIS));
		topContainer.setBorder(BorderFactory.createEmptyBorder(0,10,10,10));


		JPanel formContainer = new JPanel();
		formContainer.setLayout(new FlowLayout(FlowLayout.LEADING, 15, 30));

		formContainer.add(symbolLabel);
		formContainer.add(symbolComboBox);
		formContainer.add(quantityLabel);
		formContainer.add(quantitySpinner);

		formContainer.add(orderTypeLabel);
		formContainer.add(orderTypeComboBox);
		formContainer.add(priceLabel);
		formContainer.add(priceSpinner);

		topContainer.add(formContainer);

		JPanel submitContainer = new JPanel();
		submitContainer.setLayout(new BoxLayout(submitContainer, BoxLayout.LINE_AXIS));
		submitContainer.add(Box.createHorizontalGlue());
		submitContainer.add(sendButton);
		submitContainer.add(Box.createHorizontalGlue());

		topContainer.add(submitContainer);

		this.add(topContainer,BorderLayout.CENTER);

		statusBar = new SlimStatusBar();
		statusBar.setSize(380, 20);
		statusBar.setPreferredSize(new Dimension(380, 20));
		this.add(statusBar,BorderLayout.PAGE_END);

		orderTypeComboBox.addItemListener(new ItemListener() {

			public void itemStateChanged(ItemEvent e) {
				if (e.getItem().toString().equals("Market Price")) {
					priceSpinner.setEnabled(false);
				}
				else {
					priceSpinner.setEnabled(true);
				}
			}

		});
	}

	public void keyTypedHandler(KeyEvent evt){
		// make sure only numeric input is accepted
		char c = evt.getKeyChar();
		if (!AbstractOrdersFrame.isCharValid(evt, c)) {
			getToolkit().beep();
			evt.consume();
		}
	}
	
	/**
	 * Verifies if the character typed in the text box for quantity or price is valid.
	 * This method is used in {@link AbstractOrdersFrame} and in {@link UpdateFrame}.
	 * 
	 * @param evt the key-press event
	 * @param c the character to verify
	 * @return true if the character is valid or false otherwise
	 */
	public static boolean isCharValid(KeyEvent evt, char c) {
		boolean valid = true;
		
		// if char is not accepted, make "valid" false
		if(!(Character.isDigit(c) ||
				(c == KeyEvent.VK_BACK_SPACE) ||
				(c == KeyEvent.VK_DELETE) ||
				(c == KeyEvent.VK_PERIOD)
		)) valid = false;
		
		// if char is a dot and we already have a dot, make "valid" false
		if( c == KeyEvent.VK_PERIOD && ((JFormattedTextField) evt.getSource() ).getText().indexOf(".") > -1) valid = false;
		
		return valid;
	}

	/*
	 * Checks if form values are valid
	 * */
	protected Boolean formIsValid() {
		if (symbolComboBox.getSelectedIndex() >= 0) return true;
		else return false;
	}


	/*
	 * Creates a new order and sends it to OMS for further processing
	 * */
	protected void sendOrder(Enums.Side side) 
        {
            if (formIsValid()) {
                Order newOrder = new Order();
                newOrder.setLastUpdate(new Date());
                newOrder.setOrderTime(new Date());
                newOrder.setClientOrderID(getNewOrderUserID());
                newOrder.setOrderQuant((Long)quantitySpinner.getValue());
                newOrder.setOrderSide(side);
                newOrder.setOrderStatus(Enums.OrderStatus.ADD);
                newOrder.setMarketStatus(Enums.MarketStatus.PENDING);
                newOrder.setOrderTime(new Date());
                if (orderTypeComboBox.getSelectedItem().toString().equals("Market Price")) {
                    newOrder.setOrderType(Enums.OrderType.MARKET_PRICE);
                    newOrder.setOrderPrice(0.0f); // set price to zero for orders at market
                } else {
                    newOrder.setOrderType(Enums.OrderType.LIMIT_PRICE);
                    newOrder.setOrderPrice((Float)priceSpinner.getValue());
                }
                newOrder.setRemainedQuant((Long)quantitySpinner.getValue());
                newOrder.setSymbol(symbolComboBox.getSelectedItem().toString());
                newOrder.setUserID(parentApplet.getLoggedUser().getUserID());

                GUIOrderSenderWithReply orderSender = null;
                try {
                    System.out.println("Sending order to OMS...");
                    orderSender = new GUIOrderSenderWithReply(TradingGUI.IMQ_ADDRESS,
                                    Enums.QueueName.ORDER_REQUEST_QUEUE.toString(),
                                    Enums.QueueName.ORDER_REPLY_QUEUE.toString(),
                                    parentApplet.getLoggedUser().getUserID()+"-AddOrder",
                                    parentApplet);

                    Order replyOrder = orderSender.sendOrderGetReply(newOrder);
                    System.out.println("Order sent. Acknowledgement received. Order Status = "+replyOrder.getOrderStatus().toString()+"; Market Status: "+replyOrder.getMarketStatus().toString());
                    parentApplet.addOrIgnoreOrder(replyOrder);
                    this.setVisible(false);

                    if (orderSender != null)
                        orderSender.close();
                } catch(Exception e) {
                    System.out.println("Order sending failed.");
                    parentApplet.showPopupError("The server is down for the moment, but you order will be processed as soon as it goes back online.");
                    System.out.println("Adding order based on ClientOrderID");
                    parentApplet.addOrIgnoreOrder(newOrder);
                    this.setVisible(false);

                    if (orderSender != null)
                        try {
                            orderSender.close();
                        } catch (JMSException ex) {
                            Logger.getLogger(AbstractOrdersFrame.class.getName()).log(Level.SEVERE, null, ex);
                        }
                } catch (Error e) {
                    parentApplet.showPopupError(e.getMessage());

                    if (orderSender != null)
                        try {
                            orderSender.close();
                        } catch (JMSException ex) {
                            Logger.getLogger(AbstractOrdersFrame.class.getName()).log(Level.SEVERE, null, ex);
                        }
                } finally {
                    if (orderSender != null) try {
                            orderSender.close();
                    } catch(Exception e1) {
                            parentApplet.showPopupError(e1.getMessage());
                    }
                }
            }
	}


	private String getNewOrderUserID() {
		Date d = new Date();
		Long nr = parentApplet.getOrderNumber();
		nr++;
		parentApplet.setOrderNumber(nr);
		DecimalFormat myFormatter = new DecimalFormat("00000000");
		String output = myFormatter.format(nr);
		String orderUserID = parentApplet.getLoggedUser().getUserID()+d.getTime()+output;
		System.out.println(orderUserID);
		return orderUserID;
	}


	protected void validateFailed() {
		sendButton.setEnabled(false);
	}

	protected void validatePassed() {
		sendButton.setEnabled(true);
	}

}
