/**
 * Title:       GUI Frame class for ASETS project (ASE Trading System)
 * Author:      Claudiu Vinte (claudiu.vinte@ase.ro) and others
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements the functionality for a component in GUI
 */
package asets.gui.frames;

import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;

import asets.gui.application.TradingGUI;

public class HelpFrame extends JFrame implements HyperlinkListener {
	private static final long serialVersionUID = 1L;
	
	protected TradingGUI parentApplet;
    private JEditorPane ep;
    private javax.swing.JScrollPane jScrollPane;
    private String htmlPage;
    
    public HelpFrame(TradingGUI parentApplet, String title, String page) {
        this.parentApplet = parentApplet;
        this.htmlPage = page;
		
        String version = parentApplet.getVersionString() != null ? " - build id: " + parentApplet.getVersionString() : "";
        
        setTitle(title + version);
        setIconImage(new javax.swing.ImageIcon(getClass().getResource("/asets/gui/resources/ASETS_logo_32x32x32.png")).getImage());
        setDefaultCloseOperation(HIDE_ON_CLOSE); // this will allow the listener to dispose of this
		setResizable(true);

        initComponents();
        setLocationRelativeTo(parentApplet);

    }

    @Override
    public void hyperlinkUpdate(HyperlinkEvent e) {
        if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {

            JEditorPane pane = (JEditorPane) e.getSource();
            URL url = e.getURL();

            if (url != null && e.getDescription().startsWith("#")) {
                pane.scrollToReference(e.getDescription().substring(1));
                return;
            }
        }
    }

    private void initComponents() {

        jScrollPane = new javax.swing.JScrollPane();
        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 712, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 383, Short.MAX_VALUE)
        );

        try {
        	HTMLEditorKit htmlEditorKit = new HTMLEditorKit();
            HTMLDocument htmlDocument = (HTMLDocument)htmlEditorKit.createDefaultDocument();
            htmlDocument.setBase(getClass().getResource(htmlPage));
            htmlEditorKit.read(getClass().getResourceAsStream(htmlPage), htmlDocument, 0);
            
            ep = new JEditorPane();
            ep.addHyperlinkListener(this);
            ep.setEditable(false);
            ep.setEditorKit(htmlEditorKit);
            ep.setDocument(htmlDocument);
            
            jScrollPane.setViewportView(ep);
            
        } catch (Exception ex) {
        	ex.printStackTrace();
            Logger.getLogger(HelpFrame.class.getName()).log(Level.SEVERE, null, ex);
        }
        
        pack();
    }

}
