package asets.gui.frames;


import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.jms.InvalidClientIDException;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.InputVerifier;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import asets.api.Control;
import asets.api.Enums;
import asets.api.User;
import asets.gui.application.TradingGUI;
import asets.gui.communication.GUILogInSenderWithReply;
import asets.gui.panels.SlimStatusBar;

/**
 *
 * @author Adi
 */
public class LoginFrame extends JDialog {

	private static final long serialVersionUID = 1L;
	private JButton loginButton;
	private JPasswordField passwordField;
	private JTextField useridField;
	private JLabel passwordLabel, useridLabel;

	private TradingGUI parentApplet;
	private InputVerifier userIDInputVerifier, passwordInputVerifier;
	private SlimStatusBar statusBar;


	public  LoginFrame(TradingGUI parentApplet) {
		this.parentApplet = parentApplet;

		setTitle("Login");
		setIconImage(new javax.swing.ImageIcon(getClass().getResource("/asets/gui/resources/ASETS_logo_32x32x32.png")).getImage());
		setDefaultCloseOperation(HIDE_ON_CLOSE); // this will allow the listener to dispose of this
		setResizable(false);

		initComponents();
		setLocationRelativeTo(parentApplet);
	}


	private void initComponents() {
		ReturnKeyListener keyListener = new ReturnKeyListener(this);

		useridLabel = new javax.swing.JLabel("User ID");
		useridLabel.setPreferredSize(new Dimension(200, 20));
		useridLabel.setLabelFor(useridField);
		useridField = new JTextField();
		useridField.setPreferredSize(new Dimension(200, 20));
		useridField.addKeyListener(keyListener);

		passwordLabel = new JLabel("Password");
		passwordLabel.setPreferredSize(new Dimension(200, 20));
		passwordLabel.setLabelFor(passwordField);
		passwordField = new JPasswordField();
		passwordField.setPreferredSize(new Dimension(200, 20));
		passwordField.addKeyListener(keyListener);


		loginButton = new javax.swing.JButton("Login");
		loginButton.setMnemonic('l');
		loginButton.setEnabled(false);
		loginButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				// this is to allow setting the status bar text
				statusBar.setLabelText("Authenticating...", true);

				SwingUtilities.invokeLater( new Runnable() {
					@Override public void run() { sendLogin(); }
				});
			}
		});

		JPanel topContainer = new JPanel();
		topContainer.setSize(250,200);
		topContainer.setPreferredSize(new Dimension(250, 200));
		topContainer.setLayout(new FlowLayout(FlowLayout.LEFT, 15,5));
		topContainer.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));

		topContainer.add(useridLabel);
		topContainer.add(useridField);
		topContainer.add(Box.createRigidArea(new Dimension(230,10)));


		topContainer.add(passwordLabel);
		topContainer.add(passwordField);
		topContainer.add(Box.createRigidArea(new Dimension(230,10)));


		JPanel submitContainer = new JPanel();
		submitContainer.setPreferredSize(new Dimension(210,20));
		submitContainer.setLayout(new BoxLayout(submitContainer, BoxLayout.LINE_AXIS));
		submitContainer.add(Box.createHorizontalGlue());
		submitContainer.add(loginButton);
		submitContainer.add(Box.createHorizontalGlue());

		topContainer.add(submitContainer);

		getContentPane().add(topContainer,BorderLayout.CENTER);


		statusBar = new SlimStatusBar();
		statusBar.setSize(380, 20);
		statusBar.setPreferredSize(new Dimension(250, 20));
		getContentPane().add(statusBar,BorderLayout.PAGE_END);


		userIDInputVerifier = new InputVerifier() {

			@Override
			public boolean verify(JComponent input) {
				if (((JTextField)input).getText().isEmpty()) {
					statusBar.setLabelText("Please enter your User ID.");
					input.setBackground(Color.pink);
					validateFailed();
					return false;
				}
				else {
					statusBar.setLabelText("");
					input.setBackground(Color.white);
					validatePassed();
					return true;
				}
			}
		};
		useridField.setInputVerifier(userIDInputVerifier);

		passwordInputVerifier = new InputVerifier() {

			@Override
			public boolean verify(JComponent input) {
				if (((JTextField)input).getText().isEmpty()) {
					statusBar.setLabelText("Please enter your password.");
					input.setBackground(Color.pink);
					validateFailed();
					return false;
				}
				else {
					statusBar.setLabelText("");
					input.setBackground(Color.white);
					validatePassed();
					return true;
				}
			}
		};
		passwordField.setInputVerifier(passwordInputVerifier);

		pack();
	}


	private Boolean validateForm() {
		if (userIDInputVerifier.verify(useridField)) {
			if (passwordInputVerifier.verify(passwordField))
				return true;
			else return false;
		}
		else return false;
	}


	private void sendLogin() {
		if (validateForm()) {

			GUILogInSenderWithReply logInSender = null;
			try {

				Control controlUser = new Control(useridField.getText(),Enums.ControlType.LOG_IN_REQUEST,String.valueOf(passwordField.getPassword()));
				logInSender = new GUILogInSenderWithReply(TradingGUI.IMQ_ADDRESS,
						Enums.QueueName.LOG_IN_REQUEST_QUEUE.toString(),
						Enums.QueueName.LOG_IN_REPLY_QUEUE.toString(),
						useridField.getText()+"-LoginRequest");

				final User replyUser = logInSender.sendControlGetReply(controlUser);

				if (replyUser != null) {
					if (replyUser.getConnectionStatus() == Enums.ConnectionStatus.LOGGED_IN && replyUser.getUserStatus() == Enums.UserStatus.ENABLED) {
						
						statusBar.setLabelText("Authenticated. Initializing personal data...", true);
						
						SwingUtilities.invokeLater( new Runnable() {
							@Override public void run() { 
								try {
									parentApplet.loginSuccessful(replyUser);
									setVisible(false);
									parentApplet.invalidate();
								} catch (InvalidClientIDException e) {
									// this happens when user is already logged in
									statusBar.setLabelText("This User ID is already connected.");
								}
							}
						});
						
					}
					else {
						System.out.println("Login request rejected. Status: "+replyUser.getConnectionStatus());
						String errorMessage = "";
						if (replyUser.getUserStatus() == Enums.UserStatus.DISABLED) {
							errorMessage = "Your user ID has been suspended.";
							if (replyUser.getConnectionStatus() == Enums.ConnectionStatus.NOT_REGISTERED) errorMessage = "Invalid User ID.";
						}
						else {
							if (replyUser.getConnectionStatus() == Enums.ConnectionStatus.NOT_REGISTERED) errorMessage = "Invalid User ID.";
							if (replyUser.getConnectionStatus() == Enums.ConnectionStatus.WRONG_CREDENTIALS) errorMessage = "Wrong password. Please try again.";
							//if (replyUser.getConnectionStatus() == Enums.ConnectionStatus.LOGGED_IN_RESET) errorMessage = "This User ID is already connected.";
						}

						statusBar.setLabelText(errorMessage);
					}
				}
				else {
					statusBar.setLabelText("Could not connect to server.");
				}

			}
			catch(Exception e) {
				parentApplet.showPopupError(e.getMessage());
			}
			finally {
				try {
					if (logInSender != null) logInSender.close();
				}
				catch(Exception ex) {
					parentApplet.showPopupError(ex.getMessage());
				}
			}
		}
	}


	private void validateFailed() {
		loginButton.setEnabled(false);
	}

	private void validatePassed() {
		if (useridField.isValid() && passwordField.isValid())
			loginButton.setEnabled(true);
	}

	/**
	 * This is used for the 2 input text boxes to detect if the 
	 * Enter/Return key was pressed. In this case perform the login
	 * 
	 * @author Alexandru
	 *
	 */
	private static class ReturnKeyListener implements KeyListener {
		private LoginFrame login = null;

		public ReturnKeyListener(LoginFrame l) {
			this.login = l;
		}

		@Override
		public void keyPressed(KeyEvent arg0) {
			if (arg0.getKeyCode() == KeyEvent.VK_ENTER) {
				login.statusBar.setLabelText("Authenticating...", true);

				// this is to allow setting the status bar text
				SwingUtilities.invokeLater( new Runnable() {
					@Override public void run() { login.sendLogin(); }
				});
			}
		}

		@Override public void keyReleased(KeyEvent arg0) { /* not needed */ }
		@Override public void keyTyped(KeyEvent arg0) { /* not needed */ }

	}

}
