package asets.gui.frames;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Collection;
import java.util.Iterator;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.jms.JMSException;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JSpinner.NumberEditor;

import asets.api.Enums;
import asets.api.Instrument;
import asets.api.InstrumentList;
import asets.api.Order;
import asets.gui.application.TradingGUI;
import asets.gui.communication.GUIOrderSenderWithReply;
import asets.gui.core.PriceFormat;
import asets.gui.panels.SlimStatusBar;


public class UpdateFrame extends JDialog {

	private static final long serialVersionUID = 1L;
	
	private TradingGUI parentApplet;
	private InstrumentList instrumentList;
	private Order selectedOrder;

	private javax.swing.JComboBox orderSideComboBox, symbolComboBox,orderTypeComboBox;
	private JLabel orderSideLabel,orderTypeLabel,originalQtyLabel,priceLabel,quantityLabel,symbolLabel;
	private JSpinner originalQtyTextField,priceSpinner,quantitySpinner;
	private javax.swing.JButton sendButton;
	private SlimStatusBar statusBar;
	private long initialQuantity;
	private float initialPrice;
	private Enums.OrderType initialType;


	public UpdateFrame(TradingGUI parentApplet, Order sOrder) {
		this.parentApplet = parentApplet;
		this.selectedOrder = sOrder;
		this.instrumentList = parentApplet.getInstrumentList();
		
		setTitle("Update order");
		setIconImage(new javax.swing.ImageIcon(getClass().getResource("/asets/gui/resources/ASETS_logo_32x32x32.png")).getImage());
		setDefaultCloseOperation(DISPOSE_ON_CLOSE); // this will release the window once closed
		setResizable(false);

		initComponents();
		setLocationRelativeTo(parentApplet);
		fillForm();
	}


	private void fillForm() {
		initialQuantity = selectedOrder.getRemainedQuant();

		symbolComboBox.setSelectedItem(selectedOrder.getSymbol());

		initialType = selectedOrder.getOrderType();

		if (selectedOrder.getOrderType() == Enums.OrderType.MARKET_PRICE) {
			orderTypeComboBox.setSelectedItem("Market Price");
			orderTypeComboBox.setEnabled(false);
			priceSpinner.setValue(Float.valueOf(0.0f));
			initialPrice = Float.valueOf(0.0f);
			priceSpinner.setEnabled(false);
			
		} else {
			orderTypeComboBox.setSelectedItem("Limit Price");
			orderTypeComboBox.setEnabled(true);
			priceSpinner.setValue(selectedOrder.getOrderPrice());
			initialPrice = selectedOrder.getOrderPrice();
			priceSpinner.setEnabled(true);
		}
		
		quantitySpinner.setValue(selectedOrder.getRemainedQuant());

		originalQtyTextField.setValue(selectedOrder.getOrderQuant());
		if (selectedOrder.getOrderSide() == Enums.Side.BUY) orderSideComboBox.setSelectedItem("Buy");
		else orderSideComboBox.setSelectedItem("Sell");

	}


	private Boolean validateForm() {
		statusBar.setLabelText("");
		if (initialPrice == (Float)priceSpinner.getValue() && initialQuantity == (Long)quantitySpinner.getValue()) {
			if ((initialType == Enums.OrderType.LIMIT_PRICE && orderTypeComboBox.getSelectedItem().toString().equals("Limit Price")) ||
					(initialType == Enums.OrderType.MARKET_PRICE && orderTypeComboBox.getSelectedItem().toString().equals("Market Price"))
			) {
				statusBar.setLabelText("There are no changes for this order.");
				return false;
			}
			else return true;
		}
		else return true;
	}

	private void submitForm() {
		if (validateForm()) {
			Order requestOrder =selectedOrder;
			if(orderTypeComboBox.getSelectedItem().toString().equals("Market Price")) {
                            requestOrder.setOrderType(Enums.OrderType.MARKET_PRICE);
                            requestOrder.setOrderPrice(0.0f); // set price to zero for orders at market
			} else {
                            requestOrder.setOrderType(Enums.OrderType.LIMIT_PRICE);
                            requestOrder.setOrderPrice((Float)priceSpinner.getValue());
			}
			
			requestOrder.setRemainedQuant((Long)quantitySpinner.getValue());
			requestOrder.setOrderStatus(Enums.OrderStatus.UPDATE);

			GUIOrderSenderWithReply orderSender=null;
			try {
                            System.out.println("Sending update order request...");
                            orderSender =new GUIOrderSenderWithReply
                            (TradingGUI.IMQ_ADDRESS,
                                            asets.api.Enums.QueueName.ORDER_REQUEST_QUEUE.toString(),
                                            asets.api.Enums.QueueName.ORDER_REPLY_QUEUE.toString(),
                                            requestOrder.getUserID()+"-UdpateOrder",
                                            parentApplet);
                            Order replyOrder = orderSender.sendOrderGetReply(requestOrder);
                            System.out.println("Update acknowledgement received.");
                            parentApplet.updateOrder(replyOrder);
                            this.dispose();

                            if (orderSender != null)
                                orderSender.close();
			} catch(Exception ex) {
                            System.out.println("Update request failed.");
                            parentApplet.showPopupError(ex.getMessage());

                            if (orderSender != null)
                                try {
                                    orderSender.close();
                                } catch (JMSException ex1) {
                                    Logger.getLogger(UpdateFrame.class.getName()).log(Level.SEVERE, null, ex1);
                                }
			} finally {
                            if (orderSender != null) 
                                try {
                                    orderSender.close();
                                } catch(Exception ex2) {
                                    parentApplet.showPopupError(ex2.getMessage());
                                }
			}
		}
	}


	/**
	 * Creates all form items
	 */
	private void initComponents() {

		symbolLabel = new javax.swing.JLabel("Symbol");
		symbolLabel.setPreferredSize(new Dimension(90, 20));
		symbolComboBox = new javax.swing.JComboBox();
		symbolComboBox.setPreferredSize(new Dimension(90, 20));
		
		orderSideLabel = new javax.swing.JLabel("Side");
		orderSideLabel.setPreferredSize(new Dimension(100, 20));
		
		orderSideComboBox = new javax.swing.JComboBox();
		orderSideComboBox.setPreferredSize(new Dimension(60, 20));
		orderSideComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Sell", "Buy" }));
		orderSideComboBox.setEnabled(false);
		
		originalQtyLabel = new javax.swing.JLabel("Original quantity");
		originalQtyLabel.setPreferredSize(new Dimension(90, 20));
		originalQtyTextField = new JSpinner();
		originalQtyTextField.setPreferredSize(new Dimension(90, 20));
		originalQtyTextField.setEnabled(false);
		
		quantityLabel = new javax.swing.JLabel("Remaining quantity");
		quantityLabel.setPreferredSize(new Dimension(100, 20));
		quantitySpinner = new JSpinner();
		quantitySpinner.setPreferredSize(new Dimension(60, 20));
		quantitySpinner.setModel(new javax.swing.SpinnerNumberModel(Long.valueOf(1), Long.valueOf(1), (Long)(selectedOrder.getRemainedQuant()), Long.valueOf(1)));
		((NumberEditor) quantitySpinner.getEditor()).getFormat().setGroupingUsed(false);
        
		orderTypeLabel = new javax.swing.JLabel("Order type");
		orderTypeLabel.setPreferredSize(new Dimension(90, 20));
		orderTypeComboBox = new javax.swing.JComboBox();
		orderTypeComboBox.setPreferredSize(new Dimension(90, 20));
		orderTypeComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Market Price", "Limit Price" }));
		
                priceLabel = new javax.swing.JLabel("Price");
		priceLabel.setPreferredSize(new Dimension(100, 20));
		priceSpinner = new JSpinner();
		priceSpinner.setPreferredSize(new Dimension(75, 20));
		priceSpinner.setModel(new javax.swing.SpinnerNumberModel(Float.valueOf(1.0f), Float.valueOf(0.0001f), null, Float.valueOf(0.01f)));
		JSpinner.NumberEditor ne = new JSpinner.NumberEditor(priceSpinner);
		PriceFormat.updateFormat(ne.getFormat());
		priceSpinner.setEditor(ne);
		ne.getTextField().setText( ne.getFormat().format(1) ); // hack to get the right format displayed the first time
		
		sendButton = new javax.swing.JButton("Send");
		sendButton.setMnemonic('s');
		
		sendButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				submitForm();
			}
		});

		((JSpinner.DefaultEditor)quantitySpinner.getEditor()).getTextField().addKeyListener( new KeyListener() {
			public void keyTyped(KeyEvent evt) { keyTypedHandler(evt); }
			public void keyPressed(KeyEvent arg0) { /* no implementation */ }
			public void keyReleased(KeyEvent key) { /* no implementation */ }
		});

		((JSpinner.DefaultEditor)priceSpinner.getEditor()).getTextField().addKeyListener( new KeyListener() {
			public void keyTyped(KeyEvent evt) { keyTypedHandler(evt); }
			public void keyPressed(KeyEvent arg0) { /* no implementation */ }
			public void keyReleased(KeyEvent key) { /* no implementation */ }
		});

		JPanel topContainer = new JPanel();
		topContainer.setSize(460,200);
		topContainer.setPreferredSize(new Dimension(460, 200));
		topContainer.setLayout(new BoxLayout(topContainer, BoxLayout.PAGE_AXIS));
		topContainer.setBorder(BorderFactory.createEmptyBorder(0,10,10,10));


		JPanel formContainer = new JPanel();
		formContainer.setLayout(new FlowLayout(FlowLayout.LEADING, 15, 30));

		formContainer.add(symbolLabel);
		formContainer.add(symbolComboBox);
		formContainer.add(orderSideLabel);
		formContainer.add(orderSideComboBox);
		formContainer.add(originalQtyLabel);
		formContainer.add(originalQtyTextField);
		formContainer.add(quantityLabel);
		formContainer.add(quantitySpinner);

		formContainer.add(orderTypeLabel);
		formContainer.add(orderTypeComboBox);
		formContainer.add(priceLabel);
		formContainer.add(priceSpinner);

		topContainer.add(formContainer);

		JPanel submitContainer = new JPanel();
		submitContainer.setLayout(new BoxLayout(submitContainer, BoxLayout.LINE_AXIS));
		submitContainer.add(Box.createHorizontalGlue());
		submitContainer.add(sendButton);
		submitContainer.add(Box.createHorizontalGlue());

		topContainer.add(submitContainer);

		getContentPane().add(topContainer,BorderLayout.CENTER);

		statusBar = new SlimStatusBar();
		statusBar.setSize(460, 20);
		statusBar.setPreferredSize(new Dimension(460, 20));
		getContentPane().add(statusBar,BorderLayout.PAGE_END);


		Collection<Instrument> c = instrumentList.getInstrumentList().values();
		String[] symbols = new String[c.size()];
		int i=0;
		Iterator<Instrument> it = c.iterator();
		while (it.hasNext()) {
			symbols[i] = it.next().getSymbol();
			i++;
		}
		ComboBoxModel cbM = new DefaultComboBoxModel(symbols);
		symbolComboBox.setModel(cbM);
		symbolComboBox.setEnabled(false);

		orderTypeComboBox.addItemListener(new ItemListener() {

			public void itemStateChanged(ItemEvent e) {
				if (e.getItem().toString().equals("Market Price")) {
					priceSpinner.setEnabled(false);
					priceSpinner.setBackground(Color.white);
				} else {
					priceSpinner.setEnabled(true);
				}
			}
		});

		pack();
	}


	public void validateFailed() {
		sendButton.setEnabled(false);
	}

	public void validatePassed() {
		sendButton.setEnabled(true);
	}

	public void keyTypedHandler(KeyEvent evt){
		// make sure only numeric input is accepted
		char c = evt.getKeyChar();
		if (!AbstractOrdersFrame.isCharValid(evt, c)) {
			getToolkit().beep();
			evt.consume();
		}
	}

}
