/**
 * Title:       GUI Helper class for ASETS project (ASE Trading System)
 * Author:      Claudiu Vinte (claudiu.vinte@ase.ro) and others
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Provides a various functionality for GUI
 */

package asets.gui.menus;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;

import asets.api.Control;
import asets.api.Enums;
import asets.gui.application.TradingGUI;
import asets.gui.communication.GUILogOutSenderNoReply;
import asets.gui.core.LaF;
import asets.gui.frames.BuyFrame;
import asets.gui.frames.HelpFrame;
import asets.gui.frames.InstrumentsFrame;
import asets.gui.frames.LoginFrame;
import asets.gui.frames.MarketDepthFrame;
import asets.gui.frames.PortfolioFrame;
import asets.gui.frames.SellFrame;
import asets.gui.frames.TradesFrame;


/**
 *
 * @author Adi
 */
public class MainMenu extends JMenuBar{

	private static final long serialVersionUID = 1L;
	private JMenu sessionMenu, ordersMenu, accountMenu, marketDataMenu, optionsMenu, helpMenu;
	private JMenuItem loginMenuItem, logoutMenuItem, buyMenuItem, portfolioMenuItem, tradeMenuItem, marketOrdersMenuItem, instrumentsMenuItem, sellMenuItem, helpGuideItem, helpAboutItem;
	private TradingGUI parentApplet;
	private LoginFrame loginFrame;
	private BuyFrame buyFrame;
	private SellFrame sellFrame;
	private Map<Integer,MarketDepthFrame> marketDepthFrames;
	private PortfolioFrame portfolioFrame;
	private TradesFrame tradesFrame;
	private InstrumentsFrame instrumentsFrame;
	private HelpFrame helpGuideFrame, helpAboutFrame;
	private int mrktDpthCnt = 0;

	public MainMenu(TradingGUI parentApplet) {
		this.parentApplet = parentApplet;
		initMenu();

		marketDepthFrames = new HashMap<Integer, MarketDepthFrame>();
	}

	public void initMenu() {
		sessionMenu = new JMenu("Session");
		sessionMenu.setMnemonic('s');

		loginMenuItem = new JMenuItem("Login");
		loginMenuItem.setMnemonic('i');
		loginMenuItem.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				if (loginFrame == null) { 
					loginFrame = new LoginFrame(parentApplet);
					loginFrame.addComponentListener(new ComponentAdapter() {

						@Override
						public void componentHidden(ComponentEvent e) {
							super.componentHidden(e);
							loginFrame.dispose(); // release window from memory
							loginFrame = null;
						}

					});
				}
				loginFrame.setVisible(true);
			}
		});
		sessionMenu.add(loginMenuItem);

		logoutMenuItem = new JMenuItem("Logout");
		logoutMenuItem.setMnemonic('o');
		logoutMenuItem.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				GUILogOutSenderNoReply logOutSender = null;
				try {
					Control controlUser = new Control(parentApplet.getLoggedUser().getUserID(), Enums.ControlType.LOG_OUT_REQUEST,null);
					logOutSender = new GUILogOutSenderNoReply(TradingGUI.IMQ_ADDRESS, Enums.QueueName.LOG_OUT_REQUEST_QUEUE.toString(), parentApplet.getLoggedUser().getUserID()+"-LogoutRequest");
					logOutSender.sendLogOut(controlUser);
                                        
                                        if (logOutSender != null) 
                                            logOutSender.close();
                                        
				} catch(Exception er) {
					parentApplet.showPopupError(er.getMessage());

				} finally {
					try {
                                            if (logOutSender != null) 
                                                logOutSender.close();
					} catch(Exception er1) {
						parentApplet.showPopupError(er1.getMessage());
					}
				}
				parentApplet.logoutSuccessful(); 
			}
		});
		sessionMenu.add(logoutMenuItem);


		// ########################################## //
		ordersMenu = new JMenu("Orders");
		ordersMenu.setMnemonic('o');

		buyMenuItem = new JMenuItem("Buy");
		buyMenuItem.setMnemonic('b');
		buyMenuItem.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				if (buyFrame == null) {
					buyFrame = new BuyFrame(parentApplet);
					buyFrame.addComponentListener(new ComponentAdapter() {

						@Override
						public void componentHidden(ComponentEvent e) {
							super.componentHidden(e);
							buyFrame.dispose(); // release window from memory
							buyFrame = null;
						}

					});
				}
				buyFrame.setVisible(true);

			}
		});
		ordersMenu.add(buyMenuItem);

		sellMenuItem = new JMenuItem("Sell");
		sellMenuItem.setMnemonic('s');
		sellMenuItem.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				if (sellFrame == null) {
					sellFrame = new SellFrame(parentApplet);
					sellFrame.addComponentListener(new ComponentAdapter() {

						@Override
						public void componentHidden(ComponentEvent e) {
							super.componentHidden(e);
							sellFrame.dispose(); // release window from memory
							sellFrame = null;
						}

					});
				}
				sellFrame.setVisible(true);
			}
		});
		ordersMenu.add(sellMenuItem);


		// ########################################## //
		accountMenu = new JMenu("Account");
		accountMenu.setMnemonic('a');

		portfolioMenuItem = new JMenuItem("Portfolio");
		portfolioMenuItem.setMnemonic('p');
		portfolioMenuItem.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				if (portfolioFrame == null) {
					portfolioFrame = new PortfolioFrame(parentApplet);
					portfolioFrame.addComponentListener(new ComponentAdapter() {

						@Override
						public void componentHidden(ComponentEvent e) {
							super.componentHidden(e);
							portfolioFrame.dispose(); // release window from memory
							portfolioFrame = null;
						}

					});
				}
				portfolioFrame.setVisible(true);
			}
		});
		accountMenu.add(portfolioMenuItem);

		tradeMenuItem = new JMenuItem("Trades");
		tradeMenuItem.setMnemonic('t');
		tradeMenuItem.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				if (tradesFrame == null) {
					tradesFrame = new TradesFrame(parentApplet);
					tradesFrame.addComponentListener(new ComponentAdapter() {

						@Override
						public void componentHidden(ComponentEvent e) {
							super.componentHidden(e);
							tradesFrame.dispose(); // release window from memory
							tradesFrame = null;
						}

					});
				}
				tradesFrame.setVisible(true);
			}
		});
		accountMenu.add(tradeMenuItem);


		// ########################################## //
		marketDataMenu = new JMenu("Market Data");
		marketDataMenu.setMnemonic('m');

		instrumentsMenuItem = new JMenuItem("Instruments");
		instrumentsMenuItem.setMnemonic('i');
		instrumentsMenuItem.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				if( instrumentsFrame == null ) {
					instrumentsFrame = new InstrumentsFrame(parentApplet);
					instrumentsFrame.addComponentListener(new ComponentAdapter() {

						@Override
						public void componentHidden(ComponentEvent e) {
							super.componentHidden(e);
							instrumentsFrame.dispose(); // release window from memory
							instrumentsFrame = null;
						}

					});
				}

				instrumentsFrame.setVisible(true);

			}
		});
		marketDataMenu.add(instrumentsMenuItem);

		marketOrdersMenuItem = new JMenuItem("Market Depth");
		marketOrdersMenuItem.setMnemonic('d');
		marketOrdersMenuItem.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				final MarketDepthFrame marketDepthFrame = new MarketDepthFrame(parentApplet, mrktDpthCnt);
				marketDepthFrame.addComponentListener(new ComponentAdapter() {

					@Override
					public void componentHidden(ComponentEvent e) {
						super.componentHidden(e);
						marketDepthFrame.dispose(); // release window from memory
						marketDepthFrames.remove( marketDepthFrame.getFrameCount() );
					}

				});
				marketDepthFrames.put(mrktDpthCnt++, marketDepthFrame);
				marketDepthFrame.setVisible(true);
			}
		});
		marketDataMenu.add(marketOrdersMenuItem);


		// ########################################## //
		optionsMenu = new JMenu("Options");
		optionsMenu.setMnemonic('p');
		optionsMenu.add(LaF.buildLookAndFeelMenu());

		// ########################################## //
		helpMenu = new JMenu("Help");
		helpMenu.setMnemonic('h');

		helpGuideItem = new JMenuItem("User's Guide");
		helpGuideItem.setMnemonic('g');
		helpGuideItem.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				if (helpGuideFrame == null) {
					helpGuideFrame = new HelpFrame(parentApplet, "User's Guide", "/asets/gui/resources/guide.html");
					helpGuideFrame.addComponentListener(new ComponentAdapter() {

						@Override
						public void componentHidden(ComponentEvent e) {
							super.componentHidden(e);
							helpGuideFrame.dispose(); // release window from memory
							helpGuideFrame = null;
						}

					});
				}
				helpGuideFrame.setVisible(true);
			}
		});
		helpMenu.add(helpGuideItem);

		helpAboutItem = new JMenuItem("About ASETS");
		helpAboutItem.setMnemonic('a');
		helpAboutItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if (helpAboutFrame == null) {
					helpAboutFrame = new HelpFrame(parentApplet, "About ASETS", "/asets/gui/resources/about.html");
					helpAboutFrame.addComponentListener(new ComponentAdapter() {

						@Override
						public void componentHidden(ComponentEvent e) {
							super.componentHidden(e);
							helpAboutFrame.dispose(); // release window from memory
							helpAboutFrame = null;
						}

					});
				}
				helpAboutFrame.setVisible(true);
			}
		});
		helpMenu.add(helpAboutItem);


		// add the menu items
		this.add(sessionMenu);
		this.add(ordersMenu);
		this.add(accountMenu);
		this.add(marketDataMenu);
		this.add(optionsMenu);
		this.add(helpMenu);

		// add the version string
		String versionString = parentApplet.getVersionString();

		JLabel version = new JLabel( "ASETS ");
		version.setEnabled(false);
		if( versionString != null ) {
			version.setToolTipText("build id: " + versionString);
		}

		JPanel versionHolder = new JPanel(new BorderLayout());
		versionHolder.setOpaque(false);
		versionHolder.add(version, BorderLayout.EAST);

		this.add(versionHolder);
		

		// configure the items
		loginMenuItem.setEnabled(true);
		logoutMenuItem.setEnabled(false);
		ordersMenu.setEnabled(false);
		buyMenuItem.setEnabled(false);
		sellMenuItem.setEnabled(false);
		accountMenu.setEnabled(false);
		marketDataMenu.setEnabled(false);
	}


	public void loginSuccessful() {
		loginMenuItem.setEnabled(false);
		logoutMenuItem.setEnabled(true);
		ordersMenu.setEnabled(true);
		buyMenuItem.setEnabled(true);
		sellMenuItem.setEnabled(true);
		accountMenu.setEnabled(true);
		marketDataMenu.setEnabled(true);
	}


	public void logoutSuccessful() {
		loginMenuItem.setEnabled(true);
		logoutMenuItem.setEnabled(false);
		ordersMenu.setEnabled(false);
		buyMenuItem.setEnabled(false);
		sellMenuItem.setEnabled(false);
		accountMenu.setEnabled(false);
		marketDataMenu.setEnabled(false);

		if (loginFrame != null) 	{ loginFrame.dispose(); loginFrame = null; }
		if (buyFrame != null) 		{ buyFrame.dispose(); 	buyFrame = null; }
		if (sellFrame != null) 		{ sellFrame.dispose();  sellFrame = null; }
		if (portfolioFrame != null) { portfolioFrame.dispose(); portfolioFrame = null; }
		if (instrumentsFrame != null) { instrumentsFrame.dispose(); instrumentsFrame = null; }
		if (tradesFrame != null) 	{ tradesFrame.dispose(); tradesFrame = null; }
		for(Map.Entry<Integer, MarketDepthFrame> e: marketDepthFrames.entrySet()) {
			e.getValue().closeSubscriber();
			e.getValue().dispose();
		}
		marketDepthFrames.clear();
		mrktDpthCnt = 0;
	}

	public MarketDepthFrame getMarketDepthFrame(int count) {
		return marketDepthFrames.get(count);
	}

	public JMenu getSessionMenu() {
		return sessionMenu;
	}

}
