/**
 * Title:       GUI Helper class for ASETS project (ASE Trading System)
 * Author:      Claudiu Vinte (claudiu.vinte@ase.ro) and others
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Provides a various functionality for GUI
 */

package asets.gui.menus;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.jms.JMSException;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;

import asets.api.Enums.OrderStatus;
import asets.api.Order;
import asets.gui.application.TradingGUI;
import asets.gui.communication.GUIOrderSenderWithReply;
import asets.gui.frames.UpdateFrame;

/**
 *
 * @author Adi
 */
public class OrdersPopupMenu extends JPopupMenu{
	private static final long serialVersionUID = 1L;
	JMenuItem updateMenuItem,cancelMenuItem;
    TradingGUI parentApplet;
    Order clickedOrder;

    UpdateFrame updateFrame;

    public Order getClickedOrder() {
        return clickedOrder;
    }

    public void setClickedOrder(Order clickedOrder) {
        this.clickedOrder = clickedOrder;
    }

    public OrdersPopupMenu(TradingGUI parentApplet, String title) {
        super(title);
        this.parentApplet = parentApplet;
        updateFrame = null;

        initComponents();

    }

    public void setAllowUpdate(Boolean allowUpdate) {
        updateMenuItem.setEnabled(allowUpdate);
    }

    public void setAllowCancel(Boolean allowCancel) {
        cancelMenuItem.setEnabled(allowCancel);
    }

    private void initComponents() {
        updateMenuItem = new JMenuItem("Update order");
        updateMenuItem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (updateFrame != null) {
                	updateFrame.setVisible(false);
                	updateFrame.dispose();
                }
                updateFrame = new UpdateFrame(parentApplet, clickedOrder);
                updateFrame.setVisible(true);
            }
        });

        cancelMenuItem = new JMenuItem("Cancel order");
        cancelMenuItem.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
               int confirm = JOptionPane.showConfirmDialog(parentApplet, "Are you sure you want to cancel this order?", "Confirm", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE);

               if (confirm == JOptionPane.YES_OPTION) {
                    //trimitere mesaj cu cancel order
                    GUIOrderSenderWithReply orderSenderWithReply=null;
                    try {
                        Order requestOrder = clickedOrder;
                        requestOrder.setOrderStatus(OrderStatus.CANCEL);
                        //ii generam un UserID unic (timeStamp+userID)-
                        //Date time = new Date();
                        //requestOrder.setUserID(String.valueOf(time.getTime())+clickedOrder.getUserID());
                        System.out.println("Canceling order...");
                        orderSenderWithReply =new GUIOrderSenderWithReply
                                (TradingGUI.IMQ_ADDRESS,
                                 asets.api.Enums.QueueName.ORDER_REQUEST_QUEUE.toString(),
                                 asets.api.Enums.QueueName.ORDER_REPLY_QUEUE.toString(),
                                 requestOrder.getUserID()+"-CancelOrder",
                                 parentApplet);
                        Order replyOrder = orderSenderWithReply.sendOrderGetReply(requestOrder);
                        System.out.println("Cancel acknowledgement received.");
                        parentApplet.updateOrder(replyOrder);
                        //parentApplet.showStatus("Orderul a fost anulat cu succes");
                        
                        if (orderSenderWithReply != null)
                            orderSenderWithReply.close();
                    } catch(Exception ex) {
                        System.out.println("Order canceling failed.");
                        parentApplet.showPopupError(ex.getMessage());
                        
                        if (orderSenderWithReply != null)
                            try {
                                orderSenderWithReply.close();
                            } catch (JMSException ex1) {
                                Logger.getLogger(OrdersPopupMenu.class.getName()).log(Level.SEVERE, null, ex1);
                            }                        
                    } finally {
                        if (orderSenderWithReply != null) 
                            try {
                                orderSenderWithReply.close();
                            } catch(Exception ex2) {
                        parentApplet.showPopupError(ex2.getMessage());
                        }
                    }
                 //System.out.println(clickedOrder.getOrderID());
                //TO DO: send cancel request and update table
               }
            }
        });

        add(updateMenuItem);
        add(cancelMenuItem);
    }

}
