/**
 * Title:       Slim Status Bar class for ASETS project (ASE Trading System)
 * Author:      Claudiu Vinte (claudiu.vinte@ase.ro) and others
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements the functionality for a light Status Bar in GUI which
 * 				displays the status of the current operation in different windows.
 * 				This is a limited status bar, which can only display a text and
 * 				is different from the more powerful StatusBar class
 */

package asets.gui.panels;

/**
 * @author Adrian Bardan
 */
public class SlimStatusBar extends javax.swing.JPanel {
	private static final long serialVersionUID = -6042636520409267153L;
	private javax.swing.JLabel statusLabel;

	public SlimStatusBar() {
		initComponents();
	}

	/**
	 * Replaces the status text with an empty string.
	 */
	public void removeLabel() {
		statusLabel.setText("");
	}

	/**
	 * Sets the status text with black text.
	 * @param statusText
	 */
	public void setLabelText(String statusText, boolean blackText) {
		statusLabel.setForeground(new java.awt.Color(0, 0, 0));
		statusLabel.setText(statusText);
	}

	/**
	 * Sets the status text.
	 * @param statusText
	 */
	public void setLabelText(String statusText) {
		statusLabel.setForeground(new java.awt.Color(255, 0, 0));
		statusLabel.setText(statusText);
	}

	private void initComponents() {

		statusLabel = new javax.swing.JLabel();

		setBorder(javax.swing.BorderFactory.createEtchedBorder());
		setLayout(new java.awt.GridLayout(1, 0));

		statusLabel.setForeground(new java.awt.Color(255, 0, 0));
		statusLabel.setAlignmentX(5.0F);
		add(statusLabel);
	}

}
